package dev.fitko.fitconnect.client.subscriber;

import dev.fitko.fitconnect.api.domain.model.event.SubmissionState;
import dev.fitko.fitconnect.api.domain.model.event.SubmissionStatus;
import dev.fitko.fitconnect.api.domain.subscriber.ReceivedSubmission;
import org.junit.jupiter.api.Test;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.is;
import static org.junit.jupiter.api.Assertions.assertTrue;


import java.time.Instant;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

public class LatestReceivedSubmissionTest {

    @Test
    void testOrderedByLatestSubmission() {

        // Given
        final var maxOffset = 10;
        final var submittedDate = Instant.now();

        final List<ReceivedSubmission> submissions = IntStream.range(0, maxOffset)
                .mapToObj(i -> createSubmission(submittedDate, i))
                .sorted()
                .collect(Collectors.toList());

        // When
        final ReceivedSubmission lastSubmission = submissions.get(submissions.size() - 1);

        // Then
        assertThat(lastSubmission.getSubmittedAt(), is(Date.from(submittedDate.plusSeconds(maxOffset - 1))));

    }

    @Test
    void testOnSortedByConsecutiveIssuedAts(){

        final var maxOffset = 10;
        final var submittedDate = Instant.now();

        final List<ReceivedSubmission> submissions = IntStream.range(0, maxOffset)
                .mapToObj(i -> createSubmission(submittedDate, i))
                .sorted()
                .collect(Collectors.toList());


        boolean consecutiveSubmissionDates = false;

        for (int i = 0; i < submissions.size() - 1; i++) {
            consecutiveSubmissionDates = submissions.get(i+1).getSubmittedAt().toInstant().isAfter(submissions.get(i).getSubmittedAt().toInstant());
        }

        assertTrue(consecutiveSubmissionDates);
    }

    private ReceivedSubmission createSubmission(final Instant start, final int offset) {
        return ReceivedSubmission.builder()
                .submitState(new SubmissionStatus(SubmissionState.SUBMITTED, Date.from(start.plusSeconds(offset)), List.of()))
                .build();

    }
}
